﻿using System.Text;
using Microsoft.Extensions.Logging;

namespace GlaxLogger;

public class Logger : ILogger
{
    private readonly LogLevel _filterLevel;
    private readonly FileStream _allMessageLogfile, _filteredLogfile;
    private readonly ConsoleColor _defaultForegroundColor = Console.ForegroundColor;
    private readonly ConsoleColor _defaultBackgroundColor = Console.BackgroundColor;

    
    public Logger(LogLevel filteredLevel = LogLevel.Warning, string? outputFolderPath = null)
    {
        this._filterLevel = filteredLevel;
        string logFolderPath = outputFolderPath ?? Environment.CurrentDirectory;
        this._filteredLogfile = new FileStream(Path.Join(logFolderPath, $"{DateTime.Now:hh:mm:ss}-filtered.log"), FileMode.Create, FileAccess.Write, FileShare.Read); 
        this._allMessageLogfile = new FileStream(Path.Join(logFolderPath, $"{DateTime.Now:hh:mm:ss}.log"), FileMode.Create, FileAccess.Write, FileShare.Read); 
    }
    
    public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception? exception, Func<TState, Exception?, string> formatter)
    {
        string logLevelStr = logLevel.ToString()[..3].ToUpper();
        string timeStr = $"[{DateTime.UtcNow:HH:mm:ss.fff}]";
        string messageStr = formatter.Invoke(state, exception);
        string fullMessage = $"{logLevelStr} [{timeStr}] {messageStr}";
        byte[] fullMessageBytes = Encoding.UTF8.GetBytes(fullMessage);
        _allMessageLogfile.Write(fullMessageBytes, 0, fullMessageBytes.Length);
        if (!IsEnabled(logLevel))
            return;
        _filteredLogfile.Write(fullMessageBytes, 0, fullMessageBytes.Length);
        Console.ForegroundColor = ForegroundColorForLogLevel(logLevel);
        Console.BackgroundColor = BackgroundColorForLogLevel(logLevel);
        Console.Write(logLevelStr);
        Console.ResetColor();
        // ReSharper disable once LocalizableElement
        Console.Write($" {timeStr} ");
        Console.WriteLine(messageStr);
    }

    public bool IsEnabled(LogLevel logLevel)
    {
        return logLevel >= _filterLevel;
    }

    public IDisposable? BeginScope<TState>(TState state) where TState : notnull
    {
        return null;
    }
    
    private ConsoleColor ForegroundColorForLogLevel(LogLevel logLevel)
    {
        return logLevel switch
        {
            LogLevel.Error or LogLevel.Critical => ConsoleColor.Black,
            LogLevel.Debug => ConsoleColor.Black,
            LogLevel.Information => ConsoleColor.White,
            _ => _defaultForegroundColor
        };
    }
    
    private ConsoleColor BackgroundColorForLogLevel(LogLevel logLevel)
    {
        return logLevel switch
        {
            LogLevel.Error or LogLevel.Critical => ConsoleColor.Red,
            LogLevel.Debug => ConsoleColor.Yellow,
            LogLevel.Information => ConsoleColor.Black,
            _ => _defaultBackgroundColor
        };
    }

}